@@ -1,6 +1,6 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 
+import configparser
 import copy
 import datetime
 from functools import wraps
@@ -23,7 +23,6 @@ from click.exceptions import ClickException
 # pycreds has a find_executable that works in windows
 from dockerpycreds.utils import find_executable
 import six
-from six.moves import configparser
 import wandb
 from wandb import Config
 from wandb import env, util
@@ -32,8 +31,10 @@ from wandb import wandb_agent
 from wandb import wandb_sdk
 
 from wandb.apis import InternalApi, PublicApi
+from wandb.errors import ExecutionError, LaunchError
 from wandb.integration.magic import magic_install
 from wandb.sdk.launch.launch_add import _launch_add
+from wandb.sdk.lib.wburls import wburls
 
 # from wandb.old.core import wandb_dir
 import wandb.sdk.verify.verify as wandb_verify
@@ -41,12 +42,22 @@ from wandb.sync import get_run_from_path, get_runs, SyncManager, TMPDIR
 import yaml
 
 
-# Send cli logs to wandb/debug-cli.log by default and fallback to a temp dir.
+# Send cli logs to wandb/debug-cli.<username>.log by default and fallback to a temp dir.
 _wandb_dir = wandb.old.core.wandb_dir(env.get_dir())
 if not os.path.exists(_wandb_dir):
     _wandb_dir = tempfile.gettempdir()
+
+try:
+    _username = getpass.getuser()
+except KeyError:
+    # getuser() could raise KeyError in restricted environments like
+    # chroot jails or docker containers. Return user id in these cases.
+    _username = str(os.getuid())
+
+_wandb_log_path = os.path.join(_wandb_dir, f"debug-cli.{_username}.log")
+
 logging.basicConfig(
-    filename=os.path.join(_wandb_dir, "debug-cli.log"),
+    filename=_wandb_log_path,
     level=logging.INFO,
     format="%(asctime)s %(levelname)s %(message)s",
     datefmt="%Y-%m-%d %H:%M:%S",
@@ -110,7 +121,8 @@ def _get_cling_api(reset=None):
         wandb_sdk.wandb_setup._setup(_reset=True)
     if _api is None:
         # TODO(jhr): make a settings object that is better for non runs.
-        wandb.setup(settings=wandb.Settings(_cli_only_mode=True))
+        # only override the necessary setting
+        wandb.setup(settings=dict(_cli_only_mode=True))
         _api = InternalApi()
     return _api
 
@@ -209,23 +221,20 @@ def login(key, host, cloud, relogin, anonymously, no_offline=False):
     # TODO: handle no_offline
     anon_mode = "must" if anonymously else "never"
 
-    if host and not host.startswith("http"):
-        raise ClickException("host must start with http(s)://")
-
     wandb_sdk.wandb_login._handle_host_wandb_setting(host, cloud)
-    key = key[0] if len(key) > 0 else None
+    # A change in click or the test harness means key can be none...
+    key = key[0] if key is not None and len(key) > 0 else None
     if key:
         relogin = True
 
+    login_settings = dict(
+        _cli_only_mode=True, _disable_viewer=relogin, anonymous=anon_mode,
+    )
+    if host is not None:
+        login_settings["base_url"] = host
+
     try:
-        wandb.setup(
-            settings=wandb.Settings(
-                _cli_only_mode=True,
-                _disable_viewer=relogin,
-                anonymous=anon_mode,
-                base_url=host,
-            )
-        )
+        wandb.setup(settings=login_settings)
     except TypeError as e:
         wandb.termerror(str(e))
         sys.exit(1)
@@ -234,18 +243,44 @@ def login(key, host, cloud, relogin, anonymously, no_offline=False):
 
 
 @cli.command(
-    context_settings=CONTEXT, help="Run a grpc server", name="grpc-server", hidden=True
+    context_settings=CONTEXT, help="Run a wandb service", name="service", hidden=True
 )
-@click.option("--port", default=None, help="The host port to bind grpc service.")
+@click.option(
+    "--grpc-port", default=None, type=int, help="The host port to bind grpc service."
+)
+@click.option(
+    "--sock-port", default=None, type=int, help="The host port to bind socket service."
+)
+@click.option("--port-filename", default=None, help="Save allocated port to file.")
+@click.option("--address", default=None, help="The address to bind service.")
+@click.option("--pid", default=None, type=int, help="The parent process id to monitor.")
+@click.option("--debug", is_flag=True, help="log debug info")
+@click.option("--serve-sock", is_flag=True, help="use socket mode")
+@click.option("--serve-grpc", is_flag=True, help="use grpc mode")
 @display_error
-def grpc_server(project=None, entity=None, port=None):
-    _ = util.get_module(
-        "grpc",
-        required="grpc-server requires the grpcio library, run pip install wandb[grpc]",
-    )
-    from wandb.server.grpc_server import main as grpc_server
+def service(
+    grpc_port=None,
+    sock_port=None,
+    port_filename=None,
+    address=None,
+    pid=None,
+    debug=False,
+    serve_sock=False,
+    serve_grpc=False,
+):
+    from wandb.sdk.service.server import WandbServer
 
-    grpc_server(port=port)
+    server = WandbServer(
+        grpc_port=grpc_port,
+        sock_port=sock_port,
+        port_fname=port_filename,
+        address=address,
+        pid=pid,
+        debug=debug,
+        serve_sock=serve_sock,
+        serve_grpc=serve_grpc,
+    )
+    server.serve()
 
 
 @cli.command(
@@ -325,7 +360,8 @@ def init(ctx, project, entity, reset, mode):
     if not viewer:
         click.echo(
             click.style(
-                "We're sorry, there was a problem logging you in. Please send us a note at support@wandb.com and tell us how this happened.",
+                "We're sorry, there was a problem logging you in. "
+                "Please send us a note at support@wandb.com and tell us how this happened.",
                 fg="red",
                 bold=True,
             )
@@ -355,7 +391,7 @@ def init(ctx, project, entity, reset, mode):
     try:
         project = prompt_for_project(ctx, entity)
     except ClickWandbException:
-        raise ClickException("Could not find team: %s" % entity)
+        raise ClickException(f"Could not find team: {entity}")
 
     api.set_setting("entity", entity, persist=True)
     api.set_setting("project", project, persist=True)
@@ -871,20 +907,12 @@ def sweep(
         tuner.run(verbose=verbose)
 
 
-def _check_launch_imports():
-    req_string = 'wandb launch requires additional dependencies, install with pip install "wandb[launch]"'
-    _ = util.get_module("docker", required=req_string,)
-    _ = util.get_module("repo2docker", required=req_string,)
-    _ = util.get_module("chardet", required=req_string,)
-    _ = util.get_module("iso8601", required=req_string)
-
-
 @cli.command(
     help="Launch or queue a job from a uri (Experimental). A uri can be either a wandb "
     "uri of the form https://wandb.ai/<entity>/<project>/runs/<run_id>, "
     "or a git uri pointing to a remote repository, or path to a local directory.",
 )
-@click.argument("uri")
+@click.argument("uri", nargs=1, required=False)
 @click.option(
     "--entry-point",
     "-E",
@@ -910,15 +938,6 @@ def _check_launch_imports():
     "are not in the list of arguments for an entry point will be passed to the "
     "corresponding entry point as command-line arguments in the form `--name value`",
 )
-@click.option(  # todo: maybe take these out it's confusing with the docker image stuff
-    "--docker-args",
-    "-A",
-    metavar="NAME=VALUE",
-    multiple=True,
-    help="A `docker run` argument or flag, of the form -A name=value (e.g. -A gpus=all) "
-    "or -A name (e.g. -A t). The argument will then be passed as "
-    "`docker run --name value` or `docker run --name` respectively. ",
-)
 @click.option(
     "--name",
     envvar="WANDB_NAME",
@@ -963,8 +982,7 @@ def _check_launch_imports():
     "-c",
     metavar="FILE",
     help="Path to JSON file (must end in '.json') or JSON string which will be passed "
-    "as config to the compute resource. The exact content which should be "
-    "provided is different for each execution backend. See documentation for layout of this file.",
+    "as a launch config. Dictation how the launched run will be configured. ",
 )
 @click.option(
     "--queue",
@@ -976,13 +994,36 @@ def _check_launch_imports():
     "an argument (`--queue`), defaults to queue 'default'. Else, if name supplied, specified run queue must exist under the "
     "project and entity supplied.",
 )
+@click.option(
+    "--async",
+    "run_async",
+    is_flag=True,
+    help="Flag to run the job asynchronously. Defaults to false, i.e. unless --async is set, wandb launch will wait for "
+    "the job to finish. This option is incompatible with --queue; asynchronous options when running with an agent should be "
+    "set on wandb launch-agent.",
+)
+@click.option(
+    "--resource-args",
+    "-R",
+    metavar="FILE",
+    help="Path to JSON file (must end in '.json') or JSON string which will be passed "
+    "as resource args to the compute resource. The exact content which should be "
+    "provided is different for each execution backend. See documentation for layout of this file.",
+)
+@click.option(
+    "--cuda",
+    is_flag=False,
+    flag_value=True,
+    default=None,
+    help="Flag to build an image with CUDA enabled. If reproducing a previous wandb run that ran on GPU, a CUDA-enabled image will be "
+    "built by default and you must set --cuda=False to build a CPU-only image.",
+)
 @display_error
 def launch(
     uri,
     entry_point,
     git_version,
     args_list,
-    docker_args,
     name,
     resource,
     entity,
@@ -990,6 +1031,9 @@ def launch(
     docker_image,
     config,
     queue,
+    run_async,
+    resource_args,
+    cuda,
 ):
     """
     Run a W&B run from the given URI, which can be a wandb URI or a github repo uri or a local path.
@@ -1003,30 +1047,54 @@ def launch(
     logger.info(
         f"=== Launch called with kwargs {locals()} CLI Version: {wandb.__version__}==="
     )
-    _check_launch_imports()
     from wandb.sdk.launch import launch as wandb_launch
 
     wandb.termlog(
-        "W&B launch is in an experimental state and usage APIs may change without warning. See http://wandb.me/launch"
+        f"W&B launch is in an experimental state and usage APIs may change without warning. See {wburls.get('cli_launch')}"
     )
     api = _get_cling_api()
 
-    args_dict = util._user_args_to_dict(args_list)
-    docker_args_dict = util._user_args_to_dict(docker_args)
-    if config is not None:
-        if os.path.splitext(config)[-1] == ".json":
-            with open(config, "r") as f:
-                config = json.load(f)
+    if run_async and queue is not None:
+        raise LaunchError(
+            "Cannot use both --async and --queue with wandb launch, see help for details."
+        )
+
+    # we take a string for the `cuda` arg in order to accept None values, then convert it to a bool
+    if cuda is not None:
+        # preserve cuda=None as unspecified, otherwise convert to bool
+        if cuda == "True":
+            cuda = True
+        elif cuda == "False":
+            cuda = False
         else:
-            # assume a json string
-            try:
-                config = json.loads(config)
-            except ValueError as e:
-                wandb.termerror("Invalid backend config JSON. Parse error: %s" % e)
-                raise
+            raise LaunchError(
+                "Invalid value for --cuda: '{}' is not a valid boolean.".format(cuda)
+            )
+
+    args_dict = util._user_args_to_dict(args_list)
+
+    if resource_args is not None:
+        resource_args = util.load_json_yaml_dict(resource_args)
+        if resource_args is None:
+            raise LaunchError("Invalid format for resource-args")
+    else:
+        resource_args = {}
+
+    if config is not None:
+        config = util.load_json_yaml_dict(config)
+        if config is None:
+            raise LaunchError("Invalid format for config")
     else:
         config = {}
 
+    if (
+        uri is None
+        and docker_image is None
+        and config.get("uri") is not None
+        and config.get("docker", {}).get("docker_image") is None
+    ):
+        raise LaunchError("Must pass a URI or a docker image to launch.")
+
     if queue is None:
         # direct launch
         try:
@@ -1040,16 +1108,16 @@ def launch(
                 docker_image=docker_image,
                 name=name,
                 parameters=args_dict,
-                docker_args=docker_args_dict,
                 resource=resource,
+                resource_args=resource_args,
                 config=config,
-                synchronous=resource in ("local")
-                or resource is None,  # todo currently always true
+                synchronous=(not run_async),
+                cuda=cuda,
             )
-        except wandb_launch.LaunchError as e:
+        except LaunchError as e:
             logger.error("=== %s ===", e)
             sys.exit(e)
-        except wandb_launch.ExecutionError as e:
+        except ExecutionError as e:
             logger.error("=== %s ===", e)
             sys.exit(e)
     else:
@@ -1066,12 +1134,14 @@ def launch(
             git_version,
             docker_image,
             args_dict,
+            resource_args,
+            cuda=cuda,
         )
 
 
 @cli.command(context_settings=CONTEXT, help="Run a W&B launch agent (Experimental)")
 @click.pass_context
-@click.argument("project", nargs=1)
+@click.argument("project", nargs=1, required=False)
 @click.option(
     "--entity",
     "-e",
@@ -1079,18 +1149,30 @@ def launch(
     help="The entity to use. Defaults to current logged-in user",
 )
 @click.option("--queues", "-q", default="default", help="The queue names to poll")
+@click.option(
+    "--max-jobs",
+    "-j",
+    default=None,
+    help="The maximum number of launch jobs this agent can run in parallel. Defaults to 1. Set to -1 for no upper limit",
+)
 @display_error
-def launch_agent(ctx, project=None, entity=None, queues=None):
+def launch_agent(ctx, project=None, entity=None, queues=None, max_jobs=None):
     logger.info(
         f"=== Launch-agent called with kwargs {locals()}  CLI Version: {wandb.__version__} ==="
     )
-    _check_launch_imports()
 
     from wandb.sdk.launch import launch as wandb_launch
 
     wandb.termlog(
-        "W&B launch is in an experimental state and usage APIs may change without warning. See http://wandb.me/launch"
+        f"W&B launch is in an experimental state and usage APIs may change without warning. See {wburls.get('cli_launch')}"
     )
+    if project is None:
+        project = os.environ.get("WANDB_PROJECT")
+
+        if project is None:
+            raise LaunchError(
+                "You must specify a project name or set WANDB_PROJECT environment variable."
+            )
     api = _get_cling_api()
     queues = queues.split(",")  # todo: check for none?
     if api.api_key is None:
@@ -1100,9 +1182,12 @@ def launch_agent(ctx, project=None, entity=None, queues=None):
 
     if entity is None:
         entity = api.default_entity
+    if max_jobs is None:
+        max_jobs = float(os.environ.get("WANDB_LAUNCH_MAX_JOBS", 1))
 
     wandb.termlog("Starting launch agent ✨")
-    wandb_launch.run_agent(entity, project, queues=queues)
+
+    wandb_launch.create_and_run_agent(api, entity, project, queues, max_jobs)
 
 
 @cli.command(context_settings=CONTEXT, help="Run the W&B agent")
