@@ -1,22 +1,27 @@
+import io
 import re
 import time
-from typing import Any, Optional, TYPE_CHECKING
+from typing import Any, Dict, List, Optional, TYPE_CHECKING, Union
 
-import six
 import wandb
 from wandb.sdk.lib import telemetry
+import wandb.util
 from wandb.viz import create_custom_chart
 
 if TYPE_CHECKING:
+    import numpy as np  # type: ignore
     from wandb.sdk.internal.tb_watcher import TBHistory
 
-# We have atleast the default namestep and a global step to track
-# TODO: reset this structure on wandb.join
-STEPS = {"": {"step": 0}, "global": {"step": 0, "last_log": None}}
+# We have at least the default namestep and a global step to track
+# TODO: reset this structure on wandb.finish
+STEPS: Dict[str, Dict[str, Any]] = {
+    "": {"step": 0},
+    "global": {"step": 0, "last_log": None},
+}
 # TODO(cling): Set these when tensorboard behavior is configured.
 # We support rate limited logging by setting this to number of seconds,
 # can be a floating point.
-RATE_LIMIT_SECONDS = None
+RATE_LIMIT_SECONDS: Optional[Union[float, int]] = None
 IGNORE_KINDS = ["graphs"]
 tensor_util = wandb.util.get_module("tensorboard.util.tensor_util")
 
@@ -29,10 +34,10 @@ pb = wandb.util.get_module(
 Summary = pb.Summary if pb else None
 
 
-def make_ndarray(tensor):
+def make_ndarray(tensor: Any) -> Optional["np.ndarray"]:
     if tensor_util:
         res = tensor_util.make_ndarray(tensor)
-        # Tensorboard can log generic objects and we don't want to save them
+        # Tensorboard can log generic objects, and we don't want to save them
         if res.dtype == "object":
             return None
         else:
@@ -45,7 +50,7 @@ def make_ndarray(tensor):
         return None
 
 
-def namespaced_tag(tag, namespace=""):
+def namespaced_tag(tag: str, namespace: str = "") -> str:
     if not namespace:
         return tag
     elif tag in namespace:
@@ -55,7 +60,7 @@ def namespaced_tag(tag, namespace=""):
         return namespace + "/" + tag
 
 
-def history_image_key(key, namespace=""):
+def history_image_key(key: str, namespace: str = "") -> str:
     """Converts invalid filesystem characters to _ for use in History keys.
 
     Unfortunately this means currently certain image keys will collide silently. We
@@ -66,7 +71,9 @@ def history_image_key(key, namespace=""):
     return namespaced_tag(re.sub(r"[/\\]", "_", key), namespace)
 
 
-def tf_summary_to_dict(tf_summary_str_or_pb, namespace=""):  # noqa: C901
+def tf_summary_to_dict(  # noqa: C901
+    tf_summary_str_or_pb: Any, namespace: str = ""
+) -> Optional[Dict[str, Any]]:
     """Convert a Tensorboard Summary to a dictionary
 
     Accepts a tensorflow.summary.Summary, one encoded as a string,
@@ -94,32 +101,35 @@ def tf_summary_to_dict(tf_summary_str_or_pb, namespace=""):  # noqa: C901
         # Ignore these, caller is responsible for handling None
         return None
 
-    def encode_images(img_strs, value):
+    def encode_images(_img_strs: List[bytes], _value: Any) -> None:
         try:
-            from PIL import Image
+            from PIL import Image  # type: ignore
         except ImportError:
             wandb.termwarn(
-                'Install pillow if you are logging images with Tensorboard. To install, run "pip install pillow".',
+                "Install pillow if you are logging images with Tensorboard. "
+                "To install, run `pip install pillow`.",
                 repeat=False,
             )
-            return
+            return None
 
-        if len(img_strs) == 0:
-            return
+        if len(_img_strs) == 0:
+            return None
 
-        images = []
-        for img_str in img_strs:
-            # Supports gifs from TboardX
-            if img_str.startswith(b"GIF"):
-                images.append(wandb.Video(six.BytesIO(img_str), format="gif"))
+        images: List[Union["wandb.Video", "wandb.Image"]] = []
+        for _img_str in _img_strs:
+            # Supports gifs from TensorboardX
+            if _img_str.startswith(b"GIF"):
+                images.append(wandb.Video(io.BytesIO(_img_str), format="gif"))
             else:
-                images.append(wandb.Image(Image.open(six.BytesIO(img_str))))
-        tag_idx = value.tag.rsplit("/", 1)
+                images.append(wandb.Image(Image.open(io.BytesIO(_img_str))))
+        tag_idx = _value.tag.rsplit("/", 1)
         if len(tag_idx) > 1 and tag_idx[1].isdigit():
             tag, idx = tag_idx
             values.setdefault(history_image_key(tag, namespace), []).extend(images)
         else:
-            values[history_image_key(value.tag, namespace)] = images
+            values[history_image_key(_value.tag, namespace)] = images
+
+        return None
 
     for value in summary_pb.value:
         kind = value.WhichOneof("value")
@@ -139,6 +149,8 @@ def tf_summary_to_dict(tf_summary_str_or_pb, namespace=""):  # noqa: C901
             elif plugin_name == "histograms":
                 # https://github.com/tensorflow/tensorboard/blob/master/tensorboard/plugins/histogram/summary_v2.py#L15-L26
                 ndarray = make_ndarray(value.tensor)
+                if ndarray is None:
+                    continue
                 shape = ndarray.shape
                 counts = []
                 bins = []
@@ -167,6 +179,8 @@ def tf_summary_to_dict(tf_summary_str_or_pb, namespace=""):  # noqa: C901
                         )
             elif plugin_name == "pr_curves":
                 pr_curve_data = make_ndarray(value.tensor)
+                if pr_curve_data is None:
+                    continue
                 precision = pr_curve_data[-2, :].tolist()
                 recall = pr_curve_data[-1, :].tolist()
                 # TODO: (kdg) implement spec for showing additional info in tool tips
@@ -225,18 +239,14 @@ def tf_summary_to_dict(tf_summary_str_or_pb, namespace=""):  # noqa: C901
                     values[tag] = wandb.Histogram(np_histogram=np_histogram)
                 except ValueError:
                     wandb.termwarn(
-                        'Not logging key "{}". '
-                        "Histograms must have fewer than {} bins".format(
-                            tag, wandb.Histogram.MAX_LENGTH
-                        ),
+                        f'Not logging key "{tag}". '
+                        f"Histograms must have fewer than {wandb.Histogram.MAX_LENGTH} bins",
                         repeat=False,
                     )
             else:
                 # TODO: is there a case where we can render this?
                 wandb.termwarn(
-                    'Not logging key "{}".  Found a histogram with only 2 bins.'.format(
-                        tag
-                    ),
+                    f'Not logging key "{tag}". Found a histogram with only 2 bins.',
                     repeat=False,
                 )
         # TODO(jhr): figure out how to share this between userspace and internal process or dont
@@ -259,8 +269,8 @@ def tf_summary_to_dict(tf_summary_str_or_pb, namespace=""):  # noqa: C901
     return values
 
 
-def reset_state():
-    """Internal method for resetting state, called by wandb.join"""
+def reset_state() -> None:
+    """Internal method for resetting state, called by wandb.finish()"""
     global STEPS
     STEPS = {"": {"step": 0}, "global": {"step": 0, "last_log": None}}
 
@@ -281,7 +291,7 @@ def _log(
     NOTE: This assumes that events being passed in are in chronological order
     """
     global STEPS
-    global RATE_LIMIT
+    global RATE_LIMIT_SECONDS
     # To handle multiple global_steps, we keep track of them here instead
     # of the global log
     last_step = STEPS.get(namespace, {"step": 0})
@@ -320,14 +330,16 @@ def _log(
             or timestamp - STEPS["global"]["last_log"] >= RATE_LIMIT_SECONDS
         ):
             if history is None:
-                wandb.run._log({})
+                if wandb.run is not None:
+                    wandb.run._log({})
             else:
-                history.add({}, **kwargs)
+                history.add({})
 
         STEPS["global"]["last_log"] = timestamp
 
     if history is None:
-        wandb.run._log(log_dict, commit=False)
+        if wandb.run is not None:
+            wandb.run._log(log_dict, commit=False)
     else:
         history._row_update(log_dict)
 
@@ -335,7 +347,7 @@ def _log(
 def log(tf_summary_str_or_pb: Any, step: int = 0, namespace: str = "") -> None:
     if wandb.run is None:
         raise wandb.Error(
-            "You must call `wandb.init()` before calling wandb.tensorflow.log"
+            "You must call `wandb.init()` before calling `wandb.tensorflow.log`"
         )
 
     with telemetry.context() as tel:
