@@ -1,5 +1,6 @@
 """Handle Manager."""
 
+from collections import defaultdict
 import json
 import logging
 import math
@@ -40,7 +41,6 @@ if TYPE_CHECKING:
     from wandb.proto.wandb_internal_pb2 import (
         ArtifactDoneRequest,
         MetricSummary,
-        PartialHistoryRequest,
     )
 
 
@@ -111,10 +111,10 @@ class HandleManager(object):
 
         # keep track of summary from key/val updates
         self._consolidated_summary = dict()
-        self._sampled_history = dict()
+        self._sampled_history = defaultdict(sample.UniformSampleAccumulator)
         self._partial_history = dict()
-        self._metric_defines = dict()
-        self._metric_globs = dict()
+        self._metric_defines = defaultdict(MetricRecord)
+        self._metric_globs = defaultdict(MetricRecord)
         self._metric_track = dict()
         self._metric_copy = dict()
 
@@ -199,6 +199,9 @@ class HandleManager(object):
     def handle_files(self, record: Record) -> None:
         self._dispatch_record(record)
 
+    def handle_link_artifact(self, record: Record) -> None:
+        self._dispatch_record(record)
+
     def handle_artifact(self, record: Record) -> None:
         self._dispatch_record(record)
 
@@ -224,7 +227,6 @@ class HandleManager(object):
             k = item.key
             v = json.loads(item.value_json)
             if isinstance(v, numbers.Real):
-                self._sampled_history.setdefault(k, sample.UniformSampleAccumulator())
                 self._sampled_history[k].add(v)
 
     def _update_summary_metrics(
@@ -480,28 +482,46 @@ class HandleManager(object):
         if updated:
             self._save_summary(self._consolidated_summary)
 
-    def _flush_partial_history(
-        self, partial_history: Optional["PartialHistoryRequest"] = None,
-    ) -> None:
+    def _flush_partial_history(self, step: Optional[int] = None,) -> None:
         if self._partial_history:
             history = HistoryRecord()
             for k, v in self._partial_history.items():
                 item = history.item.add()
                 item.key = k
                 item.value_json = json.dumps(v)
-            if partial_history and partial_history.HasField("step"):
-                history.step.MergeFrom(partial_history.step)
+            if step is not None:
+                history.step.num = step
             self.handle_history(Record(history=history))
             self._partial_history = {}
 
     def handle_request_partial_history(self, record: Record) -> None:
-        history_dict = proto_util.dict_from_proto_list(
-            record.request.partial_history.item
-        )
+        partial_history = record.request.partial_history
+
+        flush = None
+        if partial_history.HasField("action"):
+            flush = partial_history.action.flush
+
+        step = None
+        if partial_history.HasField("step"):
+            step = partial_history.step.num
+
+        history_dict = proto_util.dict_from_proto_list(partial_history.item)
+        if step is not None:
+            if step < self._step:
+                logger.warning(
+                    f"Step {step} < {self._step}. Dropping entry: {history_dict}."
+                )
+                return
+            elif step > self._step:
+                self._flush_partial_history()
+                self._step = step
+        elif flush is None:
+            flush = True
+
         self._partial_history.update(history_dict)
 
-        if record.request.partial_history.action.flush:
-            self._flush_partial_history(record.request.partial_history)
+        if flush:
+            self._flush_partial_history(self._step)
 
     def handle_summary(self, record: Record) -> None:
         summary = record.summary
@@ -693,13 +713,9 @@ class HandleManager(object):
     def _handle_defined_metric(self, record: Record) -> None:
         metric = record.metric
         if metric._control.overwrite:
-            self._metric_defines.setdefault(metric.name, MetricRecord()).CopyFrom(
-                metric
-            )
+            self._metric_defines[metric.name].CopyFrom(metric)
         else:
-            self._metric_defines.setdefault(metric.name, MetricRecord()).MergeFrom(
-                metric
-            )
+            self._metric_defines[metric.name].MergeFrom(metric)
 
         # before dispatching, make sure step_metric is defined, if not define it and
         # dispatch it locally first
@@ -717,13 +733,9 @@ class HandleManager(object):
     def _handle_glob_metric(self, record: Record) -> None:
         metric = record.metric
         if metric._control.overwrite:
-            self._metric_globs.setdefault(metric.glob_name, MetricRecord()).CopyFrom(
-                metric
-            )
+            self._metric_globs[metric.glob_name].CopyFrom(metric)
         else:
-            self._metric_globs.setdefault(metric.glob_name, MetricRecord()).MergeFrom(
-                metric
-            )
+            self._metric_globs[metric.glob_name].MergeFrom(metric)
         self._dispatch_record(record)
 
     def handle_metric(self, record: Record) -> None:
