@@ -28,20 +28,20 @@ from typing import TYPE_CHECKING
 import psutil
 from six.moves import queue
 import wandb
-from wandb.util import sentry_exc
+from wandb.util import sentry_exc, sentry_set_scope
 
 from . import handler
 from . import internal_util
 from . import sender
 from . import settings_static
 from . import writer
-from ..interface import interface
+from ..interface.interface_queue import InterfaceQueue
+from ..lib import tracelog
 
 
 if TYPE_CHECKING:
-    from ..interface.interface import BackendSender
-    from .settings_static import SettingsStatic
-    from typing import Any, Dict, List, Optional, Union
+    from .settings_static import SettingsDict, SettingsStatic
+    from typing import Any, List, Optional
     from six.moves.queue import Queue
     from .internal_util import RecordLoopThread
     from wandb.proto.wandb_internal_pb2 import Record, Result
@@ -52,9 +52,11 @@ logger = logging.getLogger(__name__)
 
 
 def wandb_internal(
-    settings: "Dict[str, Union[str, float]]",
+    settings: "SettingsDict",
     record_q: "Queue[Record]",
     result_q: "Queue[Result]",
+    port: int = None,
+    user_pid: int = None,
 ) -> None:
     """Internal process function entrypoint.
 
@@ -68,8 +70,12 @@ def wandb_internal(
     """
     # mark this process as internal
     wandb._set_internal_process()
+    _setup_tracelog()
     started = time.time()
 
+    # any sentry events in the internal process will be tagged as such
+    sentry_set_scope(process_context="internal")
+
     # register the exit handler only when wandb_internal is called, not on import
     @atexit.register
     def handle_exit(*args: "Any") -> None:
@@ -80,7 +86,7 @@ def wandb_internal(
     if _settings.log_internal:
         configure_logging(_settings.log_internal, _settings._log_level)
 
-    parent_pid = os.getppid()
+    user_pid = user_pid or os.getppid()
     pid = os.getpid()
 
     logger.info(
@@ -89,12 +95,15 @@ def wandb_internal(
         datetime.fromtimestamp(started),
     )
 
-    publish_interface = interface.BackendSender(record_q=record_q)
+    tracelog.annotate_queue(record_q, "record_q")
+    tracelog.annotate_queue(result_q, "result_q")
+    publish_interface = InterfaceQueue(record_q=record_q)
 
     stopped = threading.Event()
     threads: "List[RecordLoopThread]" = []
 
     send_record_q: "Queue[Record]" = queue.Queue()
+    tracelog.annotate_queue(send_record_q, "send_q")
     record_sender_thread = SenderThread(
         settings=_settings,
         record_q=send_record_q,
@@ -106,6 +115,7 @@ def wandb_internal(
     threads.append(record_sender_thread)
 
     write_record_q: "Queue[Record]" = queue.Queue()
+    tracelog.annotate_queue(write_record_q, "write_q")
     record_writer_thread = WriterThread(
         settings=_settings,
         record_q=write_record_q,
@@ -126,7 +136,7 @@ def wandb_internal(
     )
     threads.append(record_handler_thread)
 
-    process_check = ProcessCheck(settings=_settings, pid=parent_pid)
+    process_check = ProcessCheck(settings=_settings, user_pid=user_pid)
 
     for thread in threads:
         thread.start()
@@ -159,9 +169,21 @@ def wandb_internal(
             traceback.print_exception(*exc_info)
             sentry_exc(exc_info, delay=True)
             wandb.termerror("Internal wandb error: file data was not synced")
+            if settings.get("_require_service"):
+                # TODO: We can make this more graceful by returning an error to streams.py
+                # and potentially just fail the one stream.
+                os._exit(-1)
             sys.exit(-1)
 
 
+def _setup_tracelog() -> None:
+    # TODO: remove this temporary hack, need to find a better way to pass settings
+    # to the server.  for now lets just look at the environment variable we need
+    tracelog_mode = os.environ.get("WANDB_TRACELOG")
+    if tracelog_mode:
+        tracelog.enable(tracelog_mode)
+
+
 def configure_logging(log_fname: str, log_level: int, run_id: str = None) -> None:
     # TODO: we may want make prints and stdout make it into the logs
     # sys.stdout = open(settings.log_internal, "a")
@@ -212,7 +234,7 @@ class HandlerThread(internal_util.RecordLoopThread):
         stopped: "Event",
         sender_q: "Queue[Record]",
         writer_q: "Queue[Record]",
-        interface: "BackendSender",
+        interface: "InterfaceQueue",
         debounce_interval_ms: "float" = 1000,
     ) -> None:
         super(HandlerThread, self).__init__(
@@ -263,7 +285,7 @@ class SenderThread(internal_util.RecordLoopThread):
         record_q: "Queue[Record]",
         result_q: "Queue[Result]",
         stopped: "Event",
-        interface: "BackendSender",
+        interface: "InterfaceQueue",
         debounce_interval_ms: "float" = 5000,
     ) -> None:
         super(SenderThread, self).__init__(
@@ -342,14 +364,14 @@ class ProcessCheck(object):
 
     check_process_last: "Optional[float]"
 
-    def __init__(self, settings: "SettingsStatic", pid: int) -> None:
+    def __init__(self, settings: "SettingsStatic", user_pid: "Optional[int]") -> None:
         self.settings = settings
-        self.pid = pid
+        self.pid = user_pid
         self.check_process_last = None
         self.check_process_interval = settings._internal_check_process
 
     def is_dead(self) -> bool:
-        if not self.check_process_interval:
+        if not self.check_process_interval or not self.pid:
             return False
         time_now = time.time()
         if (
@@ -359,6 +381,7 @@ class ProcessCheck(object):
             return False
         self.check_process_last = time_now
 
+        # TODO(jhr): check for os.getppid on unix being 1?
         exists = psutil.pid_exists(self.pid)
         if not exists:
             logger.warning(
