@@ -5,6 +5,7 @@ import hashlib
 import os
 import re
 import shutil
+import tempfile
 import time
 from typing import (
     Any,
@@ -28,7 +29,6 @@ from wandb import env
 from wandb import util
 from wandb.apis import InternalApi, PublicApi
 from wandb.apis.public import Artifact as PublicArtifact
-from wandb.compat import tempfile as compat_tempfile
 import wandb.data_types as data_types
 from wandb.errors import CommError
 from wandb.errors.term import termlog, termwarn
@@ -66,7 +66,7 @@ _REQUEST_POOL_CONNECTIONS = 64
 
 _REQUEST_POOL_MAXSIZE = 64
 
-ARTIFACT_TMP = compat_tempfile.TemporaryDirectory("wandb-artifacts")
+ARTIFACT_TMP = tempfile.TemporaryDirectory("wandb-artifacts")
 
 
 class _AddedObj(object):
@@ -130,6 +130,7 @@ class Artifact(ArtifactInterface):
         description: Optional[str] = None,
         metadata: Optional[dict] = None,
         incremental: Optional[bool] = None,
+        use_as: Optional[str] = None,
     ) -> None:
         if not re.match(r"^[a-zA-Z0-9_\-.]+$", name):
             raise ValueError(
@@ -157,9 +158,7 @@ class Artifact(ArtifactInterface):
         self._added_objs = {}
         self._added_local_paths = {}
         # You can write into this directory when creating artifact files
-        self._artifact_dir = compat_tempfile.TemporaryDirectory(
-            missing_ok_on_cleanup=True
-        )
+        self._artifact_dir = tempfile.TemporaryDirectory()
         self._type = type
         self._name = name
         self._description = description
@@ -170,6 +169,7 @@ class Artifact(ArtifactInterface):
         self._client_id = util.generate_id(128)
         self._sequence_client_id = util.generate_id(128)
         self._cache.store_client_artifact(self)
+        self._use_as = use_as
 
         if incremental:
             self._incremental = incremental
@@ -317,6 +317,10 @@ class Artifact(ArtifactInterface):
             "Cannot set aliases on an artifact before it has been logged or in offline mode"
         )
 
+    @property
+    def use_as(self) -> Optional[str]:
+        return self._use_as
+
     @property
     def distributed_id(self) -> Optional[str]:
         return self._distributed_id
@@ -370,7 +374,7 @@ class Artifact(ArtifactInterface):
         if not os.path.isfile(local_path):
             raise ValueError("Path is not a file: %s" % local_path)
 
-        name = name or os.path.basename(local_path)
+        name = util.to_forward_slash_path(name or os.path.basename(local_path))
         digest = md5_file_b64(local_path)
 
         if is_tmp:
@@ -424,6 +428,8 @@ class Artifact(ArtifactInterface):
         max_objects: Optional[int] = None,
     ) -> Sequence[ArtifactEntry]:
         self._ensure_can_add()
+        if name is not None:
+            name = util.to_forward_slash_path(name)
 
         # This is a bit of a hack, we want to check if the uri is a of the type
         # ArtifactEntry which is a private class returned by Artifact.get_path in
@@ -450,6 +456,7 @@ class Artifact(ArtifactInterface):
 
     def add(self, obj: data_types.WBValue, name: str) -> ArtifactEntry:
         self._ensure_can_add()
+        name = util.to_forward_slash_path(name)
 
         # This is a "hack" to automatically rename tables added to
         # the wandb /media/tables directory to their sha-based name.
@@ -473,6 +480,7 @@ class Artifact(ArtifactInterface):
             data_types.Html,
             data_types.Object3D,
             data_types.Molecule,
+            data_types._SavedModel,
         ]
 
         if not any(isinstance(obj, t) for t in allowed_types):
@@ -608,7 +616,7 @@ class Artifact(ArtifactInterface):
                             tel.feature.artifact_incremental = True
                     run.log_artifact(self)
             else:
-                wandb.run.log_artifact(self)  # type: ignore
+                wandb.run.log_artifact(self)
 
     def delete(self) -> None:
         if self._logged_artifact:
@@ -667,6 +675,13 @@ class Artifact(ArtifactInterface):
         self._final = True
         self._digest = self._manifest.digest()
 
+    def json_encode(self) -> Dict[str, Any]:
+        if not self._logged_artifact:
+            raise ValueError(
+                "Cannot json encode artifact before it has been logged or in offline mode."
+            )
+        return util.artifact_to_json(self)
+
     def _ensure_can_add(self) -> None:
         if self._final:
             raise ValueError("Can't add to finalized artifact.")
@@ -676,6 +691,7 @@ class Artifact(ArtifactInterface):
     ) -> ArtifactEntry:
         digest = digest or md5_file_b64(path)
         size = os.path.getsize(path)
+        name = util.to_forward_slash_path(name)
 
         cache_path, hit, cache_open = self._cache.check_md5_obj_path(digest, size)
         if not hit:
@@ -991,7 +1007,7 @@ class __S3BucketPolicy(StoragePolicy):
         local = LocalFileHandler()
 
         self._handler = MultiHandler(
-            handlers=[s3, local,], default_handler=TrackingHandler()
+            handlers=[s3, local,], default_handler=TrackingHandler(),
         )
 
     def config(self) -> Dict[str, str]:
