@@ -4,8 +4,7 @@
 """
 
 import itertools
-import weakref
-from six.moves import reduce
+from functools import reduce
 from operator import mul
 
 from wandb import util
@@ -17,11 +16,11 @@ torch = None
 
 def nested_shape(array_or_tuple, seen=None):
     """Figures out the shape of tensors possibly embedded in tuples
-     i.e 
-     [0,0] returns (2)
-     ([0,0], [0,0]) returns (2,2)
-     (([0,0], [0,0]),[0,0]) returns ((2,2),2)
-     """
+    i.e
+    [0,0] returns (2)
+    ([0,0], [0,0]) returns (2,2)
+    (([0,0], [0,0]),[0,0]) returns ((2,2),2)
+    """
     if seen is None:
         seen = set()
     if hasattr(array_or_tuple, "size"):
@@ -50,16 +49,14 @@ LOG_TRACK_COUNT, LOG_TRACK_THRESHOLD = range(2)
 
 
 def log_track_init(log_freq):
-    """create tracking structure used by log_track_update
-    """
+    """create tracking structure used by log_track_update"""
     l = [0] * 2
     l[LOG_TRACK_THRESHOLD] = log_freq
     return l
 
 
 def log_track_update(log_track):
-    """count (log_track[0]) up to threshold (log_track[1]), reset count (log_track[0]) and return true when reached
-    """
+    """count (log_track[0]) up to threshold (log_track[1]), reset count (log_track[0]) and return true when reached"""
     log_track[LOG_TRACK_COUNT] += 1
     if log_track[LOG_TRACK_COUNT] < log_track[LOG_TRACK_THRESHOLD]:
         return False
@@ -67,18 +64,15 @@ def log_track_update(log_track):
     return True
 
 
-class TorchHistory(object):
-    """History methods specific to PyTorch
-    """
+class TorchHistory:
+    """History methods specific to PyTorch"""
 
-    def __init__(self, history):
+    def __init__(self):
         global torch
         torch = wandb.util.get_module("torch", "Could not import torch")
-        self._history = weakref.ref(history)
         self._hook_handles = {}
         self._num_bins = 64
         self._is_cuda_histc_supported = None
-        self._jupyter_run = None
         self.hook_torch = TorchGraph.hook_torch
 
     def add_log_hooks_to_pytorch_module(
@@ -89,9 +83,8 @@ class TorchHistory(object):
         log_parameters=True,
         log_gradients=True,
         log_freq=0,
-        jupyter_run=None,
     ):
-        """ This instuments hooks into the pytorch module
+        """This instuments hooks into the pytorch module
         log_parameters - log parameters after a forward pass
         log_gradients - log gradients after a backward pass
         log_freq - log gradients/parameters every N batches
@@ -99,9 +92,6 @@ class TorchHistory(object):
         if name is not None:
             prefix = prefix + name
 
-        if jupyter_run:
-            self._jupyter_run = weakref.ref(jupyter_run)
-
         if not hasattr(module, "_wandb_hook_names"):
             module._wandb_hook_names = []
 
@@ -137,8 +127,7 @@ class TorchHistory(object):
                     )
 
     def log_tensor_stats(self, tensor, name):
-        """Add distribution statistics on a tensor's elements to the current History entry
-        """
+        """Add distribution statistics on a tensor's elements to the current History entry"""
         # TODO Handle the case of duplicate names.
 
         if isinstance(tensor, tuple) or isinstance(tensor, list):
@@ -154,16 +143,6 @@ class TorchHistory(object):
             raise TypeError(
                 "Expected Tensor, not {}.{}".format(cls.__module__, cls.__name__)
             )
-        history = self._history()
-
-        # recover history from run if using jupyter
-        if history is None and self._jupyter_run:
-            jupyter_run = self._jupyter_run()
-            if jupyter_run:
-                history = jupyter_run.history
-
-        if history is None or not history.compute:
-            return
 
         # HalfTensors on cpu do not support view(), upconvert to 32bit
         if isinstance(tensor, torch.HalfTensor):
@@ -187,7 +166,7 @@ class TorchHistory(object):
         # For pytorch 0.3 we use unoptimized numpy histograms (detach is new in 0.4)
         if not hasattr(flat, "detach"):
             tensor = flat.cpu().clone().numpy()
-            history._row_update({name: wandb.Histogram(tensor)})
+            wandb.run._log({name: wandb.Histogram(tensor)}, commit=False)
             return
 
         if flat.is_cuda:
@@ -216,12 +195,13 @@ class TorchHistory(object):
         if isinstance(flat, torch.HalfTensor):
             flat = flat.clone().type(torch.FloatTensor).detach()
 
-        # Remove nans from tensor. There's no good way to represent that in histograms.
-        flat = flat[~torch.isnan(flat)]
-        flat = flat[~torch.isinf(flat)]
-        if flat.shape == torch.Size([0]):
-            # Often the whole tensor is nan or inf. Just don't log it in that case.
+        # Skip logging if all values are nan or inf or the tensor is empty.
+        if self._no_finite_values(flat):
             return
+
+        # Remove nans and infs if present. There's no good way to represent that in histograms.
+        flat = self._remove_infs_nans(flat)
+
         tmin = flat.min().item()
         tmax = flat.max().item()
         if sparse_zeros:
@@ -257,8 +237,9 @@ class TorchHistory(object):
             tensor = torch.Tensor(tensor_np)
             bins = torch.Tensor(bins_np)
 
-        history._row_update(
-            {name: wandb.Histogram(np_histogram=(tensor.tolist(), bins.tolist()))}
+        wandb.run._log(
+            {name: wandb.Histogram(np_histogram=(tensor.tolist(), bins.tolist()))},
+            commit=False,
         )
 
     def _hook_variable_gradient_stats(self, var, name, log_track):
@@ -302,6 +283,15 @@ class TorchHistory(object):
         else:
             return handle.id in d
 
+    def _no_finite_values(self, tensor: "torch.Tensor") -> bool:
+        return tensor.shape == torch.Size([0]) or (~torch.isfinite(tensor)).all().item()
+
+    def _remove_infs_nans(self, tensor: "torch.Tensor") -> "torch.Tensor":
+        if not torch.isfinite(tensor).all():
+            tensor = tensor[torch.isfinite(tensor)]
+
+        return tensor
+
 
 class TorchGraph(wandb.data_types.Graph):
     def __init__(self):
@@ -409,7 +399,7 @@ class TorchGraph(wandb.data_types.Graph):
                 graph_hook = sub_module.register_forward_hook(
                     self.create_forward_hook(name, graph_idx)
                 )
-                wandb.run.history.torch._hook_handles[
+                wandb.run._torch._hook_handles[
                     "topology/" + str(id(graph_hook))
                 ] = graph_hook
                 if not hasattr(parent, "_wandb_hook_names"):
